#include "window.h"

#include "graphics/warning.h"
#include "input/cursor.h"
#include "input/touch.h"
#include "window/city.h"

#define MAX_QUEUE 3

static window_type window_queue[MAX_QUEUE];
static int queue_index = 0;
static window_type *current_window = 0;
static int refresh_immediate;
static int refresh_on_draw;

static void noop(void)
{
}
static void noop_mouse(const mouse *m)
{
}

static void increase_queue_index(void)
{
    queue_index++;
    if (queue_index >= MAX_QUEUE) {
        queue_index = 0;
    }
}

static void decrease_queue_index(void)
{
    queue_index--;
    if (queue_index < 0) {
        queue_index = MAX_QUEUE - 1;
    }
}

void window_invalidate(void)
{
    refresh_immediate = 1;
    refresh_on_draw = 1;
}

int window_is_invalid(void)
{
    return refresh_immediate;
}

void window_request_refresh(void)
{
    refresh_on_draw = 1;
}

int window_is(window_id id)
{
    return current_window->id == id;
}

window_id window_get_id(void)
{
    return current_window->id;
}

void window_show(const window_type *window)
{
    increase_queue_index();
    window_queue[queue_index] = *window;
    current_window = &window_queue[queue_index];
    if (!current_window->draw_background) {
        current_window->draw_background = noop;
    }
    if (!current_window->draw_foreground) {
        current_window->draw_foreground = noop;
    }
    if (!current_window->handle_mouse) {
        current_window->handle_mouse = noop_mouse;
    }
    window_invalidate();
}

void window_go_back(void)
{
    decrease_queue_index();
    current_window = &window_queue[queue_index];
    window_invalidate();
}

static void update_mouse_before(void)
{
    if (!touch_to_mouse()) {
        mouse_determine_button_state();  // touch overrides mouse
    }
}

static void update_mouse_after(void)
{
    reset_touches();
    mouse_reset_scroll();
    input_cursor_update(current_window->id);
}

void window_draw(int force)
{
    update_mouse_before();
    if (force || refresh_on_draw) {
        tooltip_invalidate();
        current_window->draw_background();
        refresh_on_draw = 0;
        refresh_immediate = 0;
    }
    current_window->draw_foreground();

    const mouse *m = mouse_get();
    current_window->handle_mouse(m);
    tooltip_handle(m, current_window->get_tooltip);
    warning_draw();
    update_mouse_after();
}
